import { ControlMetadata, ResourcedString } from "../svi-datahub";
import { ControlStates } from "../control/page";
export declare enum ControlType {
    AngularJS = "angularjs",
    Angular = "angular",
    WebComponent = "webcomponent"
}
export interface Solution {
    label: string;
    version?: string;
}
export interface SolutionExtension extends ControlMetadata {
    controlAttributes?: SolutionExtensionConfigTypeAttributes;
    metadata?: SolutionExtensionMetadata;
    resolvedDisplayName?: string;
    resolvedDescription?: string;
    clientApplication?: string;
    name: string;
}
export interface SolutionExtensionConfigTypeAttributes {
    metadata?: SolutionExtensionMetadata;
    attributes?: Record<string, SolutionExtensionAttribute>;
}
export interface SolutionExtensionAttribute {
    type: string;
    displayName: ResourcedString;
    order?: number;
    required?: boolean | string;
    localizable?: boolean | string;
    allowMultiple?: boolean | string;
    states?: Record<string, boolean>;
    defaultValue?: any;
    disabledExpression?: string;
    picklistName?: string;
    ignoreRestrictions?: boolean;
}
export interface SolutionExtensionMetadata {
    iconClass?: string;
    useDragHandle?: boolean;
    renderAs?: ControlType;
    containerRestrictions?: string[];
    fullScreenMode?: boolean;
    states?: ControlStates;
    path?: string;
    showInToolbox?: "true" | "false";
    mainDocumentOnly?: boolean;
}
export declare type SviDocumentManager = {
    key: string;
    displayName: string;
    onClick: (onFilesAdded: (files: SviLinkedFile[]) => void) => void;
};
export interface SviLinkedFile {
    fileName: string;
    link: string;
    type: string;
    content?: string;
    size: number;
}
/**
 * Functionality related to configuration.
 * Accessed from the window at window.sas.vi.config.
 */
export interface ConfigApi {
    /**
     * @method
     * @description Gets the soft expansion limit of the network nodes.
     * @returns Promise containing the soft expansion limit.
     */
    getSoftExpansionLimit(): Promise<number>;
    /**
     * @method
     * @description Gets the hard expansion limit of the network nodes.
     * @returns Promise containing the hard expansion limit.
     */
    getHardExpansionLimit(): Promise<number>;
    /**
     * @method
     * @description Gets the registered solutions and their labels and versions.
     * @returns The currently registered solutions.
     */
    getSolutions(): Map<string, Solution>;
    /**
     * @method
     * @description Registers a new solution and version in the SAS Visual Investigator about dialog box.
     * If a solution with that key already exists, it will not be overwritten.
     * @param key {string} Unique identifier for the solution.
     * @param label {string} Localised name of the solution to be shown to the user.
     * @param [version] {string} Version of the solution.
     */
    registerSolution(key: string, label: string, version?: string): void;
    /**
     * @method
     * @description Retrieves registered solution extension with a given name.
     * @param name {string} Name of the solution extension.
     * @returns A Promise that resolves to the solution extension if the name is found.
     */
    getSolutionExtension(name: string): Promise<SolutionExtension | undefined>;
    /**
     * @method
     * @description Registers a solution extension with the application.
     * @param config {SolutionExtension} Solution extension configuration.
     * @returns A Promise that resolves when the solution has been sent to the cache.
     */
    registerSolutionExtension(config: SolutionExtension): Promise<void>;
    /**
     * @method
     * @description Registers a document manager with the application.
     * @param config {SviDocumentManager} Document manager configuration.
     */
    registerDocumentManager(config: SviDocumentManager): void;
}
