import { ResourcedString } from "../svi-datahub";
import { Control } from "../control/page";
import { ChildNode, TypeAttributes } from "../object/object-api";
import { PageTemplateMetadata } from "../page-model/page-model-api";
export interface ToolbarPropertyControl {
    typeAttributes: TypeAttributes;
}
export interface PropertyConfig {
    [property: string]: any;
    order?: number;
    type: string;
    displayName: ResourcedString;
    required?: boolean | string;
    localizable?: boolean | string;
    allowMultiple?: boolean | string;
    states?: Record<string, boolean>;
    defaultValue?: any;
    disabledExpression?: string;
    picklistName?: string;
    ignoreRestrictions?: boolean;
}
export interface SelectedNode {
    id: number;
    node: ChildNode;
    parentNode?: ChildNode;
    parentSubDocument: string;
    dataSources?: {
        [property: string]: any;
    };
}
/**
 * This API is available on property editors.
 */
export interface EditorApi {
    /**
     * Registers a function to be invoked whenever one of the editor's properties changes.
     * @method
     * @param handler The function to be invoked.
     * @returns A function that stops the handler from being invoked.
     */
    onPropertyChange(handler: (category: string, property: string, currentValue: any, previousValue: any) => void): () => void;
}
/**
 * This API provides the  utility or miscellaneous functionality in SAS Visual Investigator.
 * Accessed from the window at window.sas.vi.property.
 */
export interface PropertyApi {
    /**
     * @method
     * @description Registers a custom element to be rendered as the property editor.
     * The element is rendered as a custom web component.
     * The element is passed selectedNode, property, propertyKey, and categoryKey as attributes.
     * Throws a console error if the type is already used by another registered editor.
     * @param elementName {string} Element to be rendered. For example, "custom-editor" renders an element with the tag name "custom-editor".
     * @param type {string} A custom type that is expected to be unique among registered editors.
     * @param [validatorFn] {PropertyApi~isPropertyEditorValid} Function to provide custom logic for determining if the editor is valid.
     * @param [metadataFn] {PropertyApi~getPropertyEditorMetadata} Function to provide custom metadata.
     */
    registerCustomEditor(elementName: string, type: string, validatorFn?: (propertyConfig: PropertyConfig, propertyKey: string, control: Control | ToolbarPropertyControl) => boolean, metadataFn?: (propertyConfig: PropertyConfig, propertyKey: string, control: Control | ToolbarPropertyControl) => PageTemplateMetadata): void;
    /**
     * @method
     * @description Determines whether or not the child object data source property editor is valid.
     * Typically passed as a function reference to registerCustomEditor() instead of being called directly.
     * @param {PropertyConfig} propertyConfig Configuration details relating to the property type. For example, required, type, and so on.
     * @param {string} propertyKey Key where this property's value is stored.
     * @param {Control | ToolbarPropertyControl} control The control.
     * @example
     * window.sas.vi.property.registerCustomEditor(
     *     name,
     *     type,
     *     window.sas.vi.property.validateChildObjectDataSource,
     *     window.sas.vi.property.getChildObjectMetadata)
     */
    validateChildObjectDataSource(propertyConfig: PropertyConfig, propertyKey: string, control: Control): boolean | undefined;
    /**
     * @method
     * @description Get metadata for the child object data source property editor.
     * Typically passed as a function reference to registerCustomEditor() instead of being called directly.
     * @param {PropertyConfig} propertyConfig Configuration details relating to the property type. For example, required, type, and so on.
     * @param {string} propertyKey Key where this property's value is stored.
     * @param {Control | ToolbarPropertyControl} control The control.
     * @example
     * window.sas.vi.property.registerCustomEditor(
     *     name,
     *     type,
     *     window.sas.vi.property.validateChildObjectDataSource,
     *     window.sas.vi.property.getChildObjectMetadata)
     */
    getChildObjectMetadata(propertyConfig: PropertyConfig, propertyKey: string, control: Control): PageTemplateMetadata | undefined;
}
/**
 * Determines whether or not the property editor is valid.
 * @callback PropertyApi~isPropertyEditorValid
 * @param {PropertyConfig} propertyConfig Configuration details relating to the custom property type. For example, required, type, and so on.
 * @param {string} propertyKey Key where this property's value is stored.
 * @param {Control | ToolbarPropertyControl} control The control.
 * @returns {boolean} true if valid, false if not.
 */
/**
 * Get metadata for the property editor.
 * @callback PropertyApi~getPropertyEditorMetadata
 * @param {PropertyConfig} propertyConfig Configuration details relating to the custom property type. For example, required, type, and so on.
 * @param {string} propertyKey Key where this property's value is stored.
 * @param {Control | ToolbarPropertyControl} control The control.
 * @returns {PageTemplateMetadata}
 */
